#!/usr/bin/env python

#Import gnome/gtk libraries
import gobject
import pygtk
pygtk.require('2.0')
import gtk
from gtk import gdk

#Import more advanced drawing libraries
import pango
import cairo

#And some other basic modules
from math import *
import time
import thread

#Check pygtk version
if gtk.pygtk_version < (2,9,0):
	print "PyGtk 2.9.0 or later required"
	raise SystemExit

#A lot of the implementation came from here:
#http://mikehearn.wordpress.com/2006/03/26/gtk-windows-with-alpha-channels/

#Arrow class
class ArrowWindow(gtk.Window):
	
	#Constructor
	def __init__(self, angle, size, colour):
		
		#Do all the window constructor stuff
		gtk.Window.__init__(self, gtk.WINDOW_POPUP)
		
		#Save some stuff
		self.size = size
		self.angle = angle
		self.colour = colour
		self.tip = [0,0]
		
		#Callback functions
		self.connect('delete-event', gtk.main_quit)
		self.connect('expose-event', self.expose)
		self.connect('screen-changed', self.set_colourmap)
		
		#We want to draw the window ourself
		self.set_app_paintable(True)
		
		#Don't show the window border
		self.set_decorated(False)
		
		#And cause it to draw
		self.set_colourmap()
	
	
	#Draw the window
	def set_colourmap(self, *args):
		
		#Get the colourmap
		colourmap = self.get_screen().get_rgba_colormap()
		
		#If we failed to get the colourmap
		if colourmap == None:
			print 'Your screen does not support alpha channels!'
		
		#Tell pygtk to use the colourmap, now we have found it
		self.set_colormap(colourmap)
		
		#Don't do anything else.
		return False
	
	
	#When the window is asked to draw
	def expose(self, widget, event):
	
		#Create a cairo window
		cr = widget.window.cairo_create()
	
		#Transparent background
		cr.set_source_rgba(1.0, 1.0, 1.0, 0.0)
	
		# Draw the background
		cr.set_operator(cairo.OPERATOR_SOURCE)
		cr.paint()
		
		#Arrow colour
		cr.set_source_rgba(self.colour[0], self.colour[1], self.colour[2], self.colour[3])
		
		#Draw an arrow
		points = self.get_points()
		cr.move_to(points[0][0],points[0][1])
		for p in points[1:]:
			cr.line_to(p[0],p[1])
		
		#Fill it in
		cr.fill()
		
		#Where is the tip?
		self.tip = points[0]
		
		cr.set_source_rgba(0.0, 0.2, 1.0, 1.0)
		cr.stroke()
		return False
	
	
	#The fun maths
	def get_points(self):
		#Points matrix
		points = [
			[0,150],
			[20,-55],
			[90,-35],
			[0,-150],
			[-90,-35],
			[-20,-55],
			[0,150]
		]
		
		#Scale size and angle
		size = float(self.size)/300
		angle = (float(self.angle) / (0-180)) * pi
		
		#Rotation matrix
		rotation = [
			[cos(angle), 0-sin(angle)],
			[sin(angle), cos(angle)]
		]
		
		#Multiply them together
		rotated = []
		for row in points:
			rotated.append([((row[0] * rotation[0][0]) + (row[1] * rotation[1][0]) + 150)*size, ((row[0] * rotation[0][1]) + (row[1] * rotation[1][1]) + 150)*size])
		
		#Return the rotated matrix
		return rotated

#An arrow
class Arrow:
	def __init__(self, x, y, parent=None, colour=(1.0,0.0,0.0,1.0), angle=45, size=200, lifetime=5):
		
		#Create a shaped window
		self.window=ArrowWindow(angle%360, size, colour)
		
		#Make it destroy with parent
		self.window.set_destroy_with_parent(True)
		
		#Set parent
		self.window.set_transient_for(parent)
		
		#Make it the right size
		self.window.set_default_size(size, size)
		
		#Move into position
		self.window.move(x, y)
		
		#Show it
		self.window.show_all()
		
		#Reposition, now we know where the tip is
		x -= int(self.window.tip[0])
		y -= int(self.window.tip[1])
		#window.move(x, y)
		
		#Now die
		thread.start_new_thread(self.die, (lifetime, False))
	
	def die(self, wait=5, hide=True):
		#Wait a bit
		time.sleep(wait)
		
		#Enter the gtk thread
		gtk.gdk.threads_enter()
		
		#Close the window
		if hide:
			self.window.hide()
		else:
			gtk.main_quit()
		
		#Leave the gtk thread
		gtk.gdk.threads_leave()

#Load everything
if __name__ == "__main__":
	arrow = Arrow(50,50,None,(1.0,0.0,0.0,1.0),-45, 100, 5)
	gtk.gdk.threads_init()
	gtk.main()
	
