#!/usr/bin/env python

from math import *
import cairo
import gtk
import time
import thread

class ShapedWindow(gtk.Window):
	def __init__(self, angle, size):
		gtk.Window.__init__(self, gtk.WINDOW_POPUP)
		self.size = size
		self.angle = angle
		self.tip = [0,0]
		self.connect('size-allocate', self._on_size_allocate)
		self.set_decorated(False)

	def _on_size_allocate(self, win, allocation):
		#Get height and everything needed
		w,h = allocation.width, allocation.height
		bitmap = gtk.gdk.Pixmap(None, w, h, 1)
		cr = bitmap.cairo_create()

		#Make the image completely blank (show nothing)
		cr.set_source_rgb(0.0, 0.0, 0.0)
		cr.set_operator(cairo.OPERATOR_CLEAR)
		cr.paint()

		#Draw our shape into the bitmap using cairo
		cr.set_source_rgb(1.0, 1.0, 1.0)
		cr.set_operator(cairo.OPERATOR_SOURCE)
		
		#Some reason this stops a shadow from being drawn??
		cr.arc(100, 100, 0, 0, 2 * pi)
		
		#Draw the arrow
		points = self.get_points(self.angle, self.size)
		cr.move_to(points[0][0],points[0][1])
		for p in points[1:]:
			cr.line_to(p[0],p[1])
		
		#Where is the tip?
		self.tip = points[0]
		
		#Colour
		cr.fill()

		# Set the window shape
		win.shape_combine_mask(bitmap, 0, 0)
	
	def get_points(self, angle, size=200):
		#Points matrix
		points = [
			[0,150],
			[20,-55],
			[90,-35],
			[0,-150],
			[-90,-35],
			[-20,-55],
			[0,150]
		]
		
		#Scale size and angle
		size = float(size)/300
		angle = (float(angle) / (0-180)) * pi
		
		#Rotation matrix
		rotation = [
			[cos(angle), 0-sin(angle)],
			[sin(angle), cos(angle)]
		]
		
		#Multiply them together
		rotated = []
		for row in points:
			rotated.append([((row[0] * rotation[0][0]) + (row[1] * rotation[1][0]) + 150)*size, ((row[0] * rotation[0][1]) + (row[1] * rotation[1][1]) + 150)*size])
		
		#Return the rotated matrix
		return rotated

class Arrow:
	def __init__(self, x, y, parent=None, colour='#FF0000', angle=45, size=200, lifetime=5):
		
		#Create a shaped window
		self.window=ShapedWindow(angle%360, size)
		
		#Make it destroy with parent
		self.window.set_destroy_with_parent(True)
		
		#Set parent
		self.window.set_transient_for(parent)
		
		#Make it the right size
		self.window.set_default_size(size, size)
		
		#Move into position
		self.window.move(x, y)
		
		#Colour it
		self.window.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(colour))
		
		#Show it
		self.window.show()
		
		#Reposition, now we know where the tip is
		x -= int(self.window.tip[0])
		y -= int(self.window.tip[1])
		#window.move(x, y)
		
		#Now die
		thread.start_new_thread(self.die, (lifetime, True))
	
	def die(self, wait=5, hide=True):
		#Wait a bit
		time.sleep(wait)
		
		#Enter the gtk thread
		gtk.gdk.threads_enter()
		
		#Close the window
		if hide:
			self.window.hide()
		else:
			gtk.main_quit()
		
		#Leave the gtk thread
		gtk.gdk.threads_leave()

#Load everything
if __name__ == "__main__":
	arrow = Arrow(100,50,None,'#FF0000',-45, 100, 50)
	gtk.gdk.threads_init()
	gtk.main()
	
