#!/usr/bin/env python

#Import the required modules
import sys
import os
import glob

#Cd to the right place
os.chdir(os.path.split(repr(__file__))[0][1:])

#Custom modules
import translate
import create_pot
import msgfmt

#And the basic class
class DevTools:
	
	#Constructor
	def __init__(self):
		pass
	
	
	#Welcome
	def welcome(self, args=None):
		#Print welcome string
		print "Welcome to the Ubuntu Tour development tools script."
		print "For help try     ./dev.py help."
	
	
	#Create language template
	def create_pot(self, args=None, output=True):
		#Create a pot file from all the strings
		create_pot.go(output)
	
	
	#Compile translations
	def compile_lang(self, args=None):
		#Loop throught po files
		for f in glob.glob(os.path.join(os.getcwd(),"po","*.po")):
			#Where are we saving this new file?
			out = os.path.join(translate.LOCALDIR,os.path.splitext(f.split(os.sep)[-1])[0],'LC_MESSAGES',translate.DOMAIN+'.mo')
			
			"""#Get the dir we are trying to make it in
			dir = os.sep.join(out.split(os.sep)[:-1])
			
			#If the folder does not exist, make it
			if not os.path.exists(dir):
				os.system('mkdir "%s"' % dir)"""
			
			#And compile
			msgfmt.make(f, out)
	
	
	#Commit
	def commit(self, args):
		#Check we have a message
		if len(args) < 3:
			print "Please provide a description of you changes"
			print "Useage ./dev.py commit MESSAGE"
		else:
			#Detect any language changes
			self.create_pot(args, False)
			
			#Add any newly created files
			os.system('bzr add')
			
			#Commit the changes
			os.system('bzr commit -m "%s"' % (' '.join(args[2:])))
			
			#Push the changes to the launchpad branch
			os.system('bzr push lp:ubuntu-tour/development  --remember')
	
	
	#Pull
	def update(self, args=None):
		#Get the latest code from the branch
		os.system('bzr pull lp:ubuntu-tour/development  --remember')
		
		#Recompile the language part
		os.system('sudo ./dev.py make-lang')
		#self.compile_lang()
	
	
	#Help
	def help(self, args=None):
		#Print the help stuff
		print "Welcome to the Ubuntu Tour development tools script."
		print "Useage: ./dev.py COMMAND MESSAGE"
		print
		print "Commands:"
		print "\tcreate-pot\t\tRecreate the translation templates, use after editing a tour, done automaticaly by commit or push"
		print "\tmake-lang\t\tCompiled the transated strings, done automatically by update or pull"
		print "\t\t\t\t\tDepending on where the localdir is, this may need to be run as root, be default it does"
		print "\tcommit MESSAGE\t\tSend your changes to the branch, MESSAGE should describe the changes you made"
		print "\tpush MESSAGE\t\tAlias of commit"
		print "\tupdate\t\t\tGet the newest code from the branch, always do this before you start"
		print "\tpull\t\t\tAlias of update"
		print "\thelp\t\t\tShow this message"
	
	
	#Do
	def do(self, args):
		#If we don't have isntruction, show the welcome.
		if len(args) < 2:
			self.welcome(args)
		else:
			#Nearly a switch statement
			{
				'create-pot' : self.create_pot,
				'make-lang' : self.compile_lang,
				'commit' : self.commit,
				'push' : self.commit,
				'update' : self.update,
				'pull' : self.update,
				'help' : self.help
			}.get(args[1], self.welcome)(args)


#If it is being called directly
if __name__ == "__main__":
	dev = DevTools()
	dev.do(sys.argv)
