#This file is part of Ubuntu Tour.
#
#Ubuntu Tour is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#Ubuntu Tour is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Ubuntu Tour.  If not, see <http://www.gnu.org/licenses/>.

#Created: 14/04/2011 by Anthony Stewart for Ubuntu Tour

#Imports
#Gui
import gtk
import pygtk
import webkit

#General
import os
import re
import glob
import platform
import subprocess
import time
from pprint import pprint

#Custom
import softwarecentre
import gtkcolours
from arrow import Arrow
from translate import Translate

"""
Starting the tour management again, it got to messy after too manay changes from the internal workings.
We need 3 objects:
	1: A tour manager
		This is an extended vbox, containing a webview. It can be simply thrown into any gtk window, and takes all of the html stuff out of the core program.
	2: A category item
		This is just a glorified list really
	3: A tour item
		This is simply the tour, it is fairly simple, it translated itself, it keeps track of it's page etc etc
"""

#Translations
translate = Translate()
translate.setup()
__ = translate.translate


#OS version
dist = platform.dist()


#Create a slug name
def slug(name):
	return re.sub('[\W_]+', '', name.lower())


#Start time
starttime = 0.0
def set_start(time):
	starttime = time

#Categories
categories = [
	'Hidden',
	'User Interface',
	'Applications',
	'Internet and the Cloud',
	'Files and Media',
	'Preferences and Configuration',
	'Common questions']

cat_slugs = []
for cat in categories:
	cat_slugs.append(slug(cat))


#Run a command
def terminal(command):
	p = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	return p.stdout.read()


#Parse html
def parse_html(html, extra_replacements={}):

	#Load the replacements they asked for
	for find in extra_replacements.keys():
		html = html.replace('{{%s}}' % find, extra_replacements[find])
			
	#And the distro stuff
	friendly = ['Ubuntu', 'version', 'codename']
	for i in range(len(dist)):
		html = html.replace("{{platform_" + str(i) + "}}", dist[i])
		if len(friendly) > i:
			html = html.replace("{{" + friendly[i] + "}}", dist[i])
	
	#And unity
	unity = len(terminal('ps aux | grep unity').split('\n')) > 2
	html = html.replace("{{unity}}", ("unity" if unity else "notunity"))
	
	#Return the html
	return html
		

#The tour object
class Tour:

	#Constructor
	def __init__(self, path):
		
		#Get the filepath
		self.path = path
		self.pathlist = self.path.split(os.sep)
		
		#Content isn't loaded yet
		self.content = None
		
		#Length and current page
		self.page = 1
		self.length = 0
	
	
	#Get info
	def get_info(self):
		#Try, if this fails, it's an invalid tour
		try:
			#Read the info file
			info = open(os.path.join(self.path, 'tour.info'),'r').read()
		
			#Parse the info, loop through each line in the file and store the key val pair if it exists
			self.info = {}
			for line in info.split('\n'):
				if len(line) > 2:
					if ':' in line and line[0] != '#':
						l = line.split(':',1)
						self.info[l[0]] = l[1]
		
			#Save some ot the important pars
			self.parent = self.info['parent'] if self.info['parent'] in categories else categories[6]
			self.name = self.info['name']
			self.parent_slug = slug(self.parent)
			self.name_slug = slug(self.name)
			self.parent = __(self.parent)
			self.name = __(self.name)
			
			#Description
			try:
				if not '__(' in self.info['description']:
					self.description = __(self.info['description'])
				else:
					self.description = translate.translate_string(self.info['description'])
			except:
				self.description = __("No description available")
			
			#If we have got this fair, the tour is fine
			self.valid = True

		#If it failed the tour is invalid
		except:
			self.valid = False
	
	
	#Load content
	def load_content(self, reload=False):
		#Don't load it again, if we have already got it, unless we were told to reload it
		if reload or self.content==None:
			#Find the pages
			files = glob.glob(os.path.join(self.path,"*_*.html"))
			
			#Page content, blank to begin with
			self.content = ''
			self.length = 0
			
			#Loop through the files
			for file in sorted(files):
				#Page num
				num = file.split(os.sep)[-1].split('_',1)[0]
				
				#Is it valid?
				if num.isdigit():
					
					#Read the file
					f = open(file, 'r').read()
					
					#And add it
					self.content += "<div class=\"tour-page\">" + f + "</div>"
					
					#Thats another page
					self.length += 1
			
			#Tranlate
			self.content = translate.translate_string(self.content)
			
			#Replace relative paths
			self.content = self.content.replace("~/", os.path.join(self.path, ''))
			
			#For Lightbox Effect: Search for screenshot tag and replace with img tag (with rel)
			pattern			= r'''<screenshot([^"]+)src="([^"]+)"([^>]+)rel="([^"]+)"([^>]+)'''
			pattern_string	= re.compile(pattern)
			replace_string	= r'''<a class="lightbox" href="\2" rel="\4"><img src="\2" \1\3\5></a'''
			if re.search(pattern_string, self.content):
				self.content = re.sub(pattern_string, replace_string, self.content)
			
			#For Lightbox Effect: Search for screenshot tag and replace with img tag (without rel)
			pattern			= r'''<screenshot([^"]+)src="([^"]+)"([^>]+)'''
			pattern_string	= re.compile(pattern)
			replace_string	= r'''<a class="lightbox" href="\2"><img src="\2" \1\3></a'''
			if re.search(pattern_string, self.content):
				self.content = re.sub(pattern_string, replace_string, self.content)
			
			#Parse html
			self.content = parse_html(self.content)
		
		#Return the content
		return self.content.replace("\n","")


#A category object
class Category(dict):
	
	#Constructor
	def __init__(self, uk_title):
		
		#Dictionary constructor
		dict.__init__(self)
		
		#Store the info
		self.name = __(uk_title)
		self.name_slug = slug(uk_title)
	
	
	#Load the category page
	def loadcat(self, colours):
		#Get the indexpage
		try:
			index = open('html/cat_' + self.name_slug + '.html', 'r').read()
		except:
			index = open('html/category.html', 'r').read()
		
		#Tours to omit
		omit = []
		
		#Omit tours from the ui page
		if self.name_slug == 'userinterface':
			omit += ['unitydash', 'unitylauncher', 'panel', 'desktop']
		
		#Topics
		tours = ''
		for t in self.keys():
			if not t in omit:
				tour = self[t]
				tours += '\t\t\t<li><a href="tour://' + self.name_slug + '/' + tour.name_slug + '">' + tour.name + '</a></li>\n'
		
		#Parse tours
		index = index.replace("{{TOPICS}}", tours)
		
		#Translate
		index = translate.translate_string(index)
		
		#Parse the html
		index = parse_html(index, {"CATEGORY": self.name, "TEXT-COLOUR": colours['foreground_t'][0]})
		
		#Return the text
		return index.replace("\n","")


#The tour manager
class Manager(gtk.VBox):
	
	#Constructor
	def __init__(self, parent=None, debug=False):
		
		#Load the vbox
		gtk.VBox.__init__(self)
		
		## --- MANAGER STUFF --- ##
		
		#The parent window
		self.main = parent
		
		#Window functions
		self.set_title = self.main.set_title
		self.notify = self.main.notify
		
		#Debug mode
		self.debug = debug
		
		#Not for a tablet
		self.tablet_enabled = False
		
		#Categories
		self.categories = {}
		for cat in categories:
			self.categories[slug(cat)] = Category(cat)
		
		#Get the tours
		self.get_tours()
		self.current_tour = None
		
		#Web root
		self.webroot = os.getcwd()
		self.parse_colours()
		
		#Where are we?
		self.currentpos = 'home'
		
		
		## --- GUI STUFF --- ##
		
		#Create an eventbox to put the addressbar in
		#self.addressbarbox = gtk.EventBox()
		#self.addressbarbox.set_visible_window(True)
		#self.pack_start(self.addressbarbox, expand=False)
		
		#Create the breadcrumb bar
		self.addressbar = softwarecentre.BreadCrumb([], True)
		self.addressbar.set_items([__('Home')])
		self.addressbar.set_callback("activated", self.goto)
		#self.addressbarbox.add(self.addressbar.viewport)
		self.pack_start(self.addressbar.viewport, expand=False)
		self.addressbar.viewport.modify_bg(gtk.STATE_NORMAL, self.colours['background_t'][1])
		
		#Create the webview
		self.html = webkit.WebView()
		self.html.connect("navigation-requested", self.open_url)
		self.html.connect("script-alert", self.js_alert)		
		self.loadhtml(self.loadhome())
		
		#Create a scrolled window
		self.htmlscroll = gtk.ScrolledWindow()
		self.htmlscroll.add(self.html)
		#self.htmlscroll.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
		self.htmlscroll.set_policy(gtk.POLICY_NEVER,gtk.POLICY_NEVER)
		#self.htmlscroll.set_shadow_type(gtk.SHADOW_ETCHED_IN)
		self.pack_start(self.htmlscroll, True, True)
		
		#Another vbox to hold everything else so that it can be hidden quickly
		self.extras = gtk.VBox()
		self.pack_start(self.extras, expand=False)
		
		#A progress bar
		self.progressbar = gtk.ProgressBar()
		self.extras.pack_start(self.progressbar, False, True)
		
		#Finally a box for the next/quit buttons.
		self.buttonbox = gtk.HBox()
		self.extras.pack_start(self.buttonbox, False, True)
		
		#Done
		self.done_button = gtk.Button(stock=gtk.STOCK_GO_UP)
		self.done_button.connect("clicked", self.clicked, "#up")
		self.done_button.set_size_request(150, 30)
		self.buttonbox.pack_end(self.done_button, False, True)
		
		#Next
		self.next_button = gtk.Button(stock=gtk.STOCK_GO_FORWARD)
		self.next_button.connect("clicked", self.clicked, "#forward")
		self.next_button.set_size_request(150, 30)
		self.buttonbox.pack_end(self.next_button, False, True)
		
		#Back
		self.back_button = gtk.Button(stock=gtk.STOCK_GO_BACK)
		self.back_button.connect("clicked", self.clicked, "#back")
		self.back_button.set_size_request(120, 30)
		self.buttonbox.pack_end(self.back_button, False, True)
	
	
	#Debug print
	def p(self, string, *args):
		if self.debug:
			try:
				print string, ' '.join(args)
			except (Exception):
				print string
	
	
	#Get tours
	def get_tours(self):
		
		#Even the greatest of lists has a humble beginning
		paths = []
		
		#First look in usr share, these are where most tours should be
		"""if os.path.exists(os.path.join("/", "usr","share","ubuntu-tour","tours")):
			paths += glob.glob(os.path.join("/", "usr","share","ubuntu-tour","tours","*"))
		
		#Next look in the home folder, these tours come after the /usr/share ones as they are custom imported ones so in case of a conflict, these will be read
		if os.path.exists(os.path.join(os.path.expanduser('~'), ".ubuntu-tour", "tours")):
			paths += glob.glob(os.path.join(os.path.expanduser('~'), ".ubuntu-tour", "tours", "*"))"""
		
		#Finally developement tours, these are the tours the devs are working on. Since they only exist in the bzr branch they should alway be read.
		if os.path.exists(os.path.join(os.getcwd(), "tours")):
			paths += glob.glob(os.path.join(os.getcwd(), "tours", "*"))
		
		#Now loop through the tours we've found
		for path in paths:
			#Create the tour
			tour = Tour(path)
			tour.get_info()
			
			#Check it's valid
			if not tour.valid:
				continue
			
			#Well clearly it is valid, so add it
			self.categories[tour.parent_slug][tour.name_slug] = tour
	
	
	#Navigation request
	def go(self, to, item=None):
		#Debug
		self.p(to,item)
		
		#If we are going forward, do that
		if to == "#forward":
			self.go('#' + str(self.current_tour.page + 1))
		
		#Or back
		elif to == "#back":
			self.go('#' + str(self.current_tour.page - 1))
		
		#Go back to the homepage
		elif to == "#home":
			#If we are coming from a tour, don't let categories spoil the view
			if self.currentpos == 'tour':
				self.run_js('$("#categories").html("");')
				
			#Scroll to home
			self.run_js('$("#entirety").animate({left: "0%"}, 500 );')
			self.run_js('$("#category-content").animate({top: "0%"}, 500 );')
			
			#Adjust the breadcrumbs and title
			self.addressbar.set_items([__('Home')])
			self.set_title()
			
			#Go to the home style, no buttons or progressbar
			self.set_style("home")
			self.currentpos = 'home'
		
		#Go up to the category page
		elif to == "#up":
			#Emulate selecting the previous item in the breadcrumbs
			self.goto(self.addressbar, self.addressbar.items[:2])
		
		#Go to a page
		elif to[0] == "#":
			#Make sure the page is with the limits
			page = max(min(int(to[1:]),self.current_tour.length),1)
			
			#Scroll to the page
			self.run_js('$("#tour-content").animate({top: "' + str(100 * (1-page)) + '%"}, 500 );')
			
			#Set the tours current page
			self.current_tour.page = page
			
			#If we are at the end of the tour, hide the next button and replace with an up button
			if page >= self.current_tour.length:
				self.next_button.hide()
				self.done_button.show()
			#And vice versa if we aren't
			else:
				self.next_button.show()
				self.done_button.hide()
			
			#If we are at the start of the tour, remove the back button
			if page <= 1:
				self.back_button.hide()
			#And vice versa if we aren't
			else:
				self.back_button.show()
			
			#How far through the tour are we?
			if self.current_tour.length == 1:
				progress = 1.0
			else:
				progress = (page-1.0)/(self.current_tour.length-1.0)
			
			#Update the progressbar
			self.progressbar.set_fraction(progress)
			
			#And out position
			self.currentpos = 'tour'
		
		#If we are going to a category
		elif to[0] == ".":
			cat = to[1:]
			
			#Get the category html
			self.run_js('$("#categories").html(\'' + self.loadcat(cat).replace('\\','\\\\').replace('\'','\\\'') + '\');')
			
			#Scroll to the category section
			self.run_js('$("#entirety").animate({left: "-100%"}, 500 );')
			self.run_js('$("#category-content").animate({top: "0%"}, 500 );')
			
			#Adjust the breadcrumbs and title
			self.addressbar.set_items([__('Home'),self.categories[cat].name])
			self.set_title(self.categories[cat].name)
			
			#Go to the home style, no button or progress bar
			self.set_style("home")
			
			#And out position
			self.currentpos = 'cat'
		
		#If an item was specified, we are loading a tour
		elif item != None:
			#Find the tour
			try:
				tour = self.categories[to][item]
			except:
				tour = self.categories['hidden']['notfound']
			
			#If it's not the current tour, then we need to load it
			if self.current_tour == None or tour.name != self.current_tour.name:
				self.current_tour = tour
				
				#Load the tour
				self.run_js('$("#tour-content").html(\'' + tour.load_content().replace('\\','\\\\').replace('\'','\\\'') + '\');')
				
				#Parse the lightbox
				self.run_js('$("a.lightbox").fancybox({\'titleShow\': false, \'overlayShow\': false, \'transitionIn\': \'elastic\', \'transitionOut\': \'elastic\'});')
				
				#Scrolling
				if self.tablet_enabled:
					self.run_js('$(".tour-page").overscroll({wheelDelta:100});')
				
				#Go to page 1
				self.go('#1')
			
			#If we are coming from home, don't let categories spoil the view
			if self.currentpos == 'home':
				self.run_js('$("#categories").html("");')
			
			#Scroll to the tour
			self.run_js('$("#entirety").animate({left: "-100%"}, 500 );')
			self.run_js('$("#category-content").animate({top: "-100%"}, 500 );')
			
			#Adjust the breadcrumbs and title
			self.addressbar.set_items([__('Home'),self.categories[to].name,tour.name])
			self.set_title(tour.name)
			
			#Go to the normal style, with the buttons and progresbar
			self.set_style("normal")
			
			#And out position
			self.currentpos = 'tour'
		
		#Otherwise do nothing
		else:
			pass
		
		#Return
		return True
	
	
	#Set style
	def set_style(self, style, change_bg=True):
		
		#Home style
		if style == "home":
			#Hide buttons
			self.extras.hide()
			self.displaystyle = "home"
		
		#Normal style
		else: #if style == "normal":
			self.extras.show()
			self.displaystyle = "normal"
		
		#Return
		return True
	
	
	#Breadcrumbs clicked
	def goto(self, widget, params):
		self.p('>>', ' > '.join(params))
		
		#Home, category, item
		l = len(params)
		if l == 1:
			#Go home
			self.go("#home")
		
		elif l == 2:
			#Go to category
			for k in self.categories.keys():
				if self.categories[k].name == params[1]:
					self.go("." + k)
					break
		
		elif l == 3:
			#Go to item
			for k in self.categories.keys():
				if self.categories[k].name == params[1]:
					for l in self.categories[k].keys():
						if self.categories[k][l].name == params[2]:
							self.go(k, l)
							break
					break
	
	
	#When a button is clicked
	def clicked(self, widget, param=None, *user_params):
		self.go(param)	
	
	
	#Convert url into normal
	def convert_url(self, url):
		#Replacements
		repl = {
			'%20' : ' ',
			'%5B' : '[',
			'%5D' : ']'
		}
		
		#Do the replacements
		for k in repl.keys():
			url = url.replace(k, repl[k])
		
		#Return the formatted url
		return url
	
	
	#This opens a url
	def open_url(self, view, frame, networkRequest):
		
		#Where does it want us to go?
		uri = networkRequest.get_uri()
		
		#Debug
		self.p("Request to go to %s" % uri)
		
		#Protocol
		protocol = uri.split('://')[0]
		
		#Return val
		retval = 1
		
		#Is it a normal url, let webkit load it
		if protocol in ['file','http','https','ftp']:
			retval = 0
		
		#If we are to load a category
		elif protocol == 'category':
			self.go("." + self.convert_url(uri[11:]))
		
		#If we are to load a tour
		elif protocol == 'tour':
			tour = self.convert_url(uri[7:]).split('/',1)
			self.go(tour[0],tour[1])
		
		#If it is a process
		elif protocol == 'command':
			subprocess.Popen(self.convert_url(uri[10:]), shell=debug)
		
		#If it is an arrow
		elif protocol == 'arrow':
			command = [0, 0, -45, 100, 5]
			for key, val in enumerate(uri[8:].split(',')): command[key] = int(val)
			Arrow(command[0], command[1], self.main.window, (1.0,0.0,0.0,1.0), command[2], command[3], command[4])
		
		#If we are to execute some python code
		elif protocol == 'exec':
			print "WARNING: Please do not use exec://. It is being depricated."
			exec(self.convert_url(uri[7:]))
		
		#If we are to load an external url
		elif protocol[0:3] == 'ex_':
			self.p("Protocol called was %s" % protocol)
			webbrowser.open(uri[3:])
		
		#Otherwise just let webkit deal with it
		else:
			self.p("Protocol called was %s" % protocol)
			retval = 0
		
		#Return 1 to stop any other handlers running, eg. the default uri handler...
		return retval
	
	
	#For js alerts
	def js_alert(self, view, frame, message):
		if message[0] == '$':
			try:
				exec(message[1:])
			except Exception, e:
				print e
			return 1
		else:
			return 0
	
	
	#To insert code
	def run_js(self, script):
		self.html.execute_script(script)
	
	
	#Check progression requirements
	def check_progression(self, reqs):
		#Default values
		cancontinue = True
		elsemessage = ["Progression Warning", "We recommend you run the relevant program before continuing."]
		justwarning = True
		
		#Get the ones set by the page
		exec(reqs)
		
		#Can we continue
		if cancontinue:
			return True
		
		#We are being stopped
		else:
			#Display a message
			self.notify(elsemessage[0], elsemessage[1], os.getcwd()+"/images/logo.svg")
			
			#Is this just a warning, or should we actually stop the user
			if justwarning:
				return True
			else:
				return False
	
	
	#Parse the colours
	def parse_colours(self):
		#Get the colours
		self.colours = gtkcolours.get()
		
		#Loop through them
		for k in self.colours.keys():
			#Get 64k colour hex
			hex = self.colours[k].to_string()
			
			#Convert to 256 colour hex
			if len(hex) == 13:
				hex = hex[0:3] + hex[5:7] + hex[9:11]
			
			#Set the colour
			self.colours[k] = (hex, self.colours[k])

	
	#Load the homepage
	def loadhome(self):
		#Get the indexpage
		index = open('html/index.html', 'r').read()
		
		#Translate
		index = translate.translate_string(index)
		
		#Parse colours
		home_replacements = {
			"BACK-COLOUR": self.colours['background_t'][0],
			"TEXT-COLOUR": self.colours['foreground_t'][0],
			"BACK-COLOUR-NORM": self.colours['background'][0],
			"TEXT-COLOUR-NORM": self.colours['foreground'][0]
		}
		
		#And the icons
		for n in range(1,len(self.categories)):
			cat = self.categories[cat_slugs[n]]
			home_replacements["CAT_NAME_" + str(n)] = cat.name
			home_replacements["CAT_SLUG_" + str(n) ] = cat.name_slug
		
		#Parse the html
		index = parse_html(index, home_replacements)
		
		#Return the text
		return index

	
	#Load the homepage
	def loadcat(self, cat_slug):
		#Pass it to the category to create it's own page
		return self.categories[cat_slug].loadcat(self.colours)
	
	
	#Load some html
	def loadhtml(self, html, root=None, markup="text/html"):
		#Get the rootpath if it wasn't specified
		root = 'file://' + os.path.join(self.webroot,'') if root == None else root
		
		#And output
		self.html.load_string(html, markup, "UTF-8", root)
