#!/usr/bin/env python

#This file is part of Ubuntu Tour.
#
#Ubuntu Tour is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#Ubuntu Tour is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Ubuntu Tour.  If not, see <http://www.gnu.org/licenses/>.

import pygtk
pygtk.require('2.0')
import gtk
import widgets

import os
import sys
import time
import thread
start = time.time()

#CD
try:
	os.chdir(os.path.split(repr(__file__))[0][1:])
except:
	print 'Warning: Could not change directory to file, unexpected errors may occur'
	pass

import webbrowser
import pynotify
import warnings

import tour
import dialogs
import version
import extras

#Version number
if (extras.get_flag("v") or extras.get_argument("version")):
	print "Version: " + version.Version
	sys.exit(0)

#Debug
debug = False
if (extras.get_flag("d") or extras.get_argument("debug")):
	debug = True
	print "Version: " + version.Version

#OS check
required = float("10.04")
if not (extras.check_os("Ubuntu", 10.04) or extras.get_argument("ignore-os") or debug):
	os.popen('zenity --error --text="This program requires Ubuntu %s or later."' % required)
	sys.exit("Unsupported Distribution. Ubuntu %s or later required." % required)

#If we are not in debug mode, we don't need warnings
if not debug:
	warnings.simplefilter("ignore")
	#sys.stderr = open('/dev/null','w')

#Translate function
__ = tour.translate.translate

#Set the starttime
tour.set_start(start)

#GTK Window
class Window:	
	
	#Use the notification system
	def notify(self, title, message, image=''):
		#Make a notification
		if len(image) > 0:
			notification = pynotify.Notification (title, message, image)
		else:
			notification = pynotify.Notification (title, message)
		
		#Show
		notification.show()
	
	
	#Various dialogues
	def show_dialog(self, widget, page):
		
		#Show dialog
		dialogs.Show(self, page)
		
		#File > Import
		if page == 'import':
			tour.update_menu()
	
	
	#Program asked to quit
	def delete_event(self, widget, event, data=None):
		gtk.main_quit()
		return False
	
	
	#Set the title - catch the function
	def set_title(self, title=''):
		self.window.set_title(title + (" - " if title != "" else "") + version.ProgramName)
	
	
	#Go home
	def go_home(self, *args):#*args are throwaways.
		self.manager.go('#home')
	
	
	#Function to run on load
	def __init__(self):
		#Create window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.set_title()
		self.window.set_icon_from_file(os.getcwd()+"/images/logo.svg")
		self.window.set_default_size(1000,550)
		self.window.set_position(gtk.WIN_POS_CENTER)
		self.window.connect("delete_event", self.delete_event)
		
		#Create a hbox to contain everything
		self.box = gtk.VBox()
		self.window.add(self.box)
		
		#Create a main menu
		self.mainmenu = widgets.MainMenu([
			[__('File') , [__('Import'), __('Exit')]],
			[__('Go') , [__('Home')]],
			[__('Help') , [__('About'), __('Report a bug')]]
		])

		#Menu bar connects
		self.mainmenu.connect(__('File'), __('Exit'), self.delete_event)
		self.mainmenu.connect(__('File'), __('Import'), self.show_dialog, 'import')
		self.mainmenu.connect(__('Go'), __('Home'), self.go_home)
		self.mainmenu.connect(__('Help'), __('About'), self.show_dialog, 'about')
		self.mainmenu.connect(__('Help'), __('Report a bug'), self.show_dialog, 'reportbug')
		self.box.pack_start(self.mainmenu, False, True)
		
		#Create a hbox to contain the main sections
		self.hbox = gtk.HBox()
		self.box.pack_start(self.hbox, True, True)
		
		#Load the menu buttons
		"""
		self.menuscroll = gtk.ScrolledWindow()
		self.menuscroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		self.hbox.pack_start(self.menuscroll, False, True)
		tour.make_menu(self)
		"""
		
		#Now create a vbox for the main page
		self.manager = tour.Manager(self, debug)
		self.hbox.pack_start(self.manager, True, True)
		
		#Show everything
		self.window.show_all()
		
		#Now load the first pages
		self.manager.set_style('home')
		
		#Tablet enabled?
		if (extras.get_flag("t") or extras.get_argument("touch")):
			self.manager.tablet_enabled = True
		
		#Some dialogs that might want showing
		self.importdialog = None
		self.aboutdialog = None


#Load everything
if __name__ == "__main__":
	Window()
	gtk.gdk.threads_init()
	gtk.main()
